/* ! Music List */
.music-list-view {
    height: 1fr;
}

.music-list-view-item {
    height: 4;
}

.music-list-view-item-title-label {
    height: 1;
    color: #cacaca;
}

.music-list-view-item-subtitle-label {
    height: 1;
    color: #a9a9a9;
}

.music-list-screen-add-box {
    height: 3;
}

.music-list-screen-add-input {
    width: 1fr;
}

/* ! IndeterminateProgress */
.indeterminate-progress-bar { height: 1; }

/* ! Image Label */
.image-label {
    height: 1fr;
    width: 1fr;
    align: center middle;
    text-align: center;
}

/* ! Configurate List */
.configurate-list-view {
    border: solid cadetblue;
    height: 1fr;
    width: 1fr;
}

.configurate-list-view-item {
    border: solid dodgerblue;
    background: #0000;
    border-title-color: #aaaaaa;
    border-subtitle-color: #6b6b6b;
    height: auto;
}

/* ! Any Elements CSS */
.pass-one-width { width: 1; }

/* ! Log Menu */
.log-menu {
    background: $surface;
    color: $text;
    height: 50vh;
    dock: bottom;
    layer: notes;
    border-top: hkey $primary;
    offset-y: 0;
    transition: offset 400ms in_out_cubic;
    padding: 0 1 1 1;
}

.log-menu:focus {
    offset: 0 0 !important;
}

.log-menu.-hidden {
    offset-y: 100%;
}

/* ! Nofy */
Nofy {
    layer: nofys;
    background: $background;
    margin: 2 4;
    padding: 1 2;
    width: auto;
    height: auto;
}

CallNofy {
    layer: nofys;
    background: $background;
    margin: 2 4;
    padding: 1 2;
    width: auto;
    height: auto;
}