/* ! Main Screen */
#_default  {
    layout: horizontal;
}

.screen-box {
    border: solid cornflowerblue;
    width: 1fr;
    height: 100%;
}

.test-box-border {
    border: solid cadetblue;
}

/* ! Play Screen */
.player-visual-panel {
    border: solid cadetblue;
    height: 1fr;
}

.player-contol-panel {
    border: solid cadetblue;
    height: 7;
}

.box-buttons-sound-control {
    height: 3;
}

.button-sound-control {
    width: 1fr;
}

.music-selected-label {
    width: 1fr;
    height: 1;
    align: center middle;
    text-align: center;
}
