Unknown {
    align: center middle;
    background: blue;
    color: white;
}

Unknown>Static {
    width: 70;    
}

#unknown-title {
    content-align-horizontal: center;
    text-style: reverse;
}

#unknown-any-key {
    content-align-horizontal: center;
}
